import azure.cognitiveservices.speech as speechsdk


def try_parse_float(arg: str):
    try:
        return True, float(arg)
    except ValueError:
        return False, None


def error(arg: str):
    return ValueError('Invalid value: ' + arg)


def pitch(arg: str):
    if arg.endswith('Hz') and try_parse_float(arg[:-2])[0]:
        # 1. Absolute value: 400Hz
        # 2. Relative value: +10Hz, -20Hz
        return arg
    if arg.endswith('%') and try_parse_float(arg[:-1])[0]:
        # Percentage values
        return arg
    if arg.endswith('st') and try_parse_float(arg[:-2])[0] and arg[0] in {'+', '-'}:
        # Relative value: +1st, -2st
        return arg
    if (result := try_parse_float(arg)) and result[0]:
        return result[1]
    if arg in {'default', 'x-low', 'low', 'medium', 'high', 'x-high'}:
        return arg
    raise error(arg)


def rate(arg: str):
    if arg.endswith('%') and try_parse_float(arg[:-1])[0]:
        # Percentage values
        return arg
    if arg in {"default", "x-slow", "slow", "medium", "fast", "x-fast"}:
        # enum values
        return arg
    if (result := try_parse_float(arg)) and result[0]:
        # float values that will be converted to percentages
        return result[1]
    if arg.endswith('f') and try_parse_float(arg[:-1])[0]:
        # raw float values
        return arg[:-1]
    raise error(arg)


# `format` will appear in the cli error messages, so we need to keep this name, although it shallows the builtin.
# noinspection PyShadowingBuiltins
# pylint: disable=redefined-builtin
def format(arg: str):
    if arg in speechsdk.SpeechSynthesisOutputFormat.__members__:
        return arg
    raise error(arg)



import base64
import json
import re
import requests

from . import errors
from .urls import TRAIL_URL


def _get_auth_token() -> str:
    """
    Get a trial auth token from the trial webpage.
    """
    response = requests.get(TRAIL_URL)
    if response.status_code != 200:
        raise errors.TokenRetrievalError(status_code=response.status_code)
    text = response.text

    # We don't need bs4, because a little of regex is enough.

    match = re.search(r'\s+var\s+localizedResources\s+=\s+\{((.|\n)*?)\};', text, re.M)
    retrieval_error = errors.TokenRetrievalError(message='Could not extract token from webpage.',
                                                 status_code=response.status_code)
    if match is None:
        raise retrieval_error
    token = re.search(r'\s+token:\s*"([^"]+)"', match.group(1), re.M)
    if token is None:
        raise retrieval_error
    return token.group(1)


def _parse_info_from_token(token: str) -> dict:
    """
    Parse the region from the token.
    """
    _, content, _ = token.split('.')
    # The token is base64 encoded, so we need to decode it.
    # :joy: Actually, GitHub Copilot provides the following line, and it works!
    json_string = base64.b64decode(content + '=' * (-len(content) % 4))
    return json.loads(json_string)


def get_token_info_dict() -> dict:
    """
    Get an auth token and its associated information in a dict.
    """
    token = _get_auth_token()
    info = _parse_info_from_token(token)
    info['token'] = token
    return info



"""
aspeak is a CLI tool as well as a Python library that
uses trial auth token of Azure Cognitive Services to do speech synthesis for you.
"""

from .auth import get_token_info_dict
from .token import Token
from .api import *
from .errors import AspeakError, TokenRetrievalError



from ..quality import QUALITIES


def validate_quality(args, parser):
    if not hasattr(args, 'quality'):
        return
    if hasattr(args, 'format') and args.quality != 0:
        parser.error("You can't use --quality with --format.")
    for ext in ["mp3", "ogg", "wav", "webm"]:
        if getattr(args, ext) and args.quality not in QUALITIES[ext]:
            parser.error(f"Invalid quality {args.quality} for {ext}.")


def get_ineffective_args_for_listing(args):
    result = [option for option in
              ['pitch', 'rate', 'style', 'role', 'style_degree', 'quality', 'format', 'encoding', 'file', 'text',
               'ssml'] if hasattr(args, option)]
    if args.output_path is not None:
        result.append('output_path')
    return ', '.join(result)


def has_text_options(args):
    return any(hasattr(args, option) for option in ['pitch', 'rate', 'style', 'role', 'style_degree'])



from azure.cognitiveservices.speech import SpeechSynthesisOutputFormat


def get_available_formats() -> set:
    """
    Returns a set of all available output formats.
    """
    return {
        x.name for x in SpeechSynthesisOutputFormat
        if not x.name.endswith('Siren')  # *Siren is unsupported now
    }
